import org.opencv.core.Core;
import org.opencv.core.Mat;
import org.opencv.core.Point;
import org.opencv.core.Size;
import org.opencv.videoio.VideoCapture;
import org.opencv.highgui.HighGui;
import org.opencv.imgproc.Imgproc;
import org.opencv.imgproc.Moments;
import org.opencv.videoio.Videoio;
import processing.core.PApplet;

import java.util.LinkedList;
import java.util.Queue;

public class Tracker {

    private final int CAM = 0;                  // Kamera-ID (0 für Webcam, 1 für andere Cams)
    private final int EXP = -7;                 // Belichtungszeit (idealerweise: 0)
    private final int FPS = 60;                 // FPS

    private Point currentPosition;
    public Tracker(){

    }

    public static void main(String[] args){
        Tracker tracker = new Tracker();
        tracker.run();
    }

    private void run(){
        VideoCapture capture = setUp();

        if (!capture.isOpened()) {
            System.out.println("Fehler: Die Webcam konnte nicht geöffnet werden.");
            return;
        }

        Mat frame = new Mat();

        while (true){
            if (capture.read(frame)){
                HighGui.imshow("GTCar", frame);

                currentPosition = trackPosition(frame);

                if (HighGui.waitKey(1) == 27) {
                    break;
                }
            }else{
                System.out.println("Fehler: Das Kamerabild konnte nicht gelesen werden.");
                break;
            }
        }


    }

    public int getX(){
        return (int) currentPosition.x;
    }

    public int getY(){
        return (int) currentPosition.y;
    }

    public Point getPosition(){
        return currentPosition;
    }


    private Point trackPosition(Mat thresholdFrame) {
        // Berechne die Momente des binären Bildes
        Moments moments = Imgproc.moments(thresholdFrame);

        // Prüfen, ob m00 (Fläche) nicht null ist, um Division durch Null zu vermeiden
        if (moments.get_m00() != 0) {
            int cx = (int) (moments.get_m10() / moments.get_m00());
            int cy = (int) (moments.get_m01() / moments.get_m00());
            return new Point(cx, cy);
        } else {

            return null;
        }
    }
    private VideoCapture setUp() {
        System.loadLibrary(Core.NATIVE_LIBRARY_NAME);
        VideoCapture capture = new VideoCapture(CAM);

        if (!capture.isOpened()) {
            System.out.println("Fehler: Die Webcam konnte nicht geöffnet werden.");
        }

        capture.set(Videoio.CAP_PROP_EXPOSURE, EXP);
        capture.set(Videoio.CAP_PROP_FPS, FPS);

        return capture;
    }
}
